/*******************************************************************************
  Copyright (c) 2022-2024, Intel Corporation

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

      * Redistributions of source code must retain the above copyright notice,
        this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
      * Neither the name of Intel Corporation nor the names of its contributors
        may be used to endorse or promote products derived from this software
        without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#ifndef OOO_MGR_RESET_H
#define OOO_MGR_RESET_H

IMB_DLL_LOCAL void
ooo_mgr_aes_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL void
ooo_mgr_docsis_aes_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL void
ooo_mgr_cmac_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL void
ooo_mgr_ccm_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_aes_xcbc_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_hmac_sha1_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_hmac_sha224_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_hmac_sha256_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_hmac_sha384_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_hmac_sha512_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_hmac_md5_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_zuc_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_sha1_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_sha256_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_sha512_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_des_reset(void *p_ooo_mgr, const unsigned num_lanes);

IMB_DLL_LOCAL
void
ooo_mgr_snow3g_reset(void *p_ooo_mgr, const unsigned num_lanes);

#endif /* OOO_MGR_RESET_H */
